// BongoDoc.h: interface for the CBongoDoc class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_BONGODOC_H__C21746FB_0B1D_469E_9138_809DB10A1B23__INCLUDED_)
#define AFX_BONGODOC_H__C21746FB_0B1D_469E_9138_809DB10A1B23__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

typedef enum BO_KF_PATH_MARKER_TYPE
{
	unknown_path_marker,
	dot_path_marker,
	point_path_marker,
};

typedef enum BONGO_ANIMATION_PREVIEW_MODE
{
	unknown_mode,
	forward_mode,
	backward_mode,
	ping_pong_mode,
}BO_PREVIEW_MODE;



class BO_SDK_API_EXP CBongoDoc  
{
protected:
	CBongoDoc();

public:
	//Only one CBongoDoc or CBongoView object can be instantiated at any given
	//time.  The pseudo-constructor below enforces this - it will return false
	//if there is already another object around.  In case you are interested,
	//the reason has to do with ensuring the modifications to the document level
	//animation data are undoable.

	static bool DocumentObject(CBongoDoc*& doc);	

public:
	virtual ~CBongoDoc();

/////////////////////////////////////////////////
// General

	bool	AnimateInsideBlocks() const;
	void	SetAnimateInsideBlocks(bool b);

	bool	UseProxyForMultipleObjectTransforms() const;
	void	SetUseProxyForMultipleObjectTransforms(bool b);

	bool	ShowRecticleInAnimatedViewport() const;
	void	SetShowRecticleInAnimatedViewport(bool b);


/////////////////////////////////////////////////
// Object Pivots
	bool	DisplayHierarchyLinks() const;
	void	SetDisplayHierarchyLinks(bool b);

	bool	EnableObjectPivotDisplay() const;
	void	SetEnableObjectPivotDisplay(bool b);

	bool	DisplayObjectPivotForNonSelectedNonAnimatedChildren() const;
	void	SetDisplayObjectPivotForNonSelectedNonAnimatedChildren(bool b);

	int		PivotSize() const;
	bool	SetPivotSize(int iLength);

/////////////////////////////////////////////////
// Preview
	
	int		TimelineLengthInSeconds() const;
	bool	SetTimelineLengthInSeconds(int i);

	BO_PREVIEW_MODE	PreviewMode() const;
	void	SetPreviewMode(BO_PREVIEW_MODE mode);

//////////////////////////////////////////////////
// Show pivot / view path
	bool	NeverShowPivotViewPath() const;
	void	SetNeverShowPivotViewPath(bool b);

	BO_KF_PATH_MARKER_TYPE	ShowKeyframesAndFrameMarkerType() const;
	void					SetShowKeyframesAndFrameMarkerType(BO_KF_PATH_MARKER_TYPE type);
	
	int		PointDotStepSize() const;
	bool	SetPointDotStepSize(int i);

//////////////////////////////////////////////////
// Refresh 

	//Updates the model with the new animation data on objects.
	//Moves all objects and views into position and sets all other animated parameters.
	//Expensive - use sparingly.
	void	RefreshScene() const;

	//Redraws everything on each docking panel - helpful if an operation fails
	//to update the display.
	void	RefreshTimeline() const;
	void	RefreshAnimationManager() const;

	//Rebuilds the runtime animation list
	void	Regenerate() const;

///////////////////////////////////////////////////
// Timeline
	bool	AnimationLimits(double& dStart, double& dStop) const;
	bool	SetAnimationLimits(double dStart, double dStop);

	double	TimelineSliderPosition() const;
	bool	SetTimelineSliderPosition( double d );

	int		TimelineTicks() const;
	bool	SetTimelineTicks(int iTicks);

protected:
	LPVOID m_pBuffer;
};

#endif // !defined(AFX_BONGODOC_H__C21746FB_0B1D_469E_9138_809DB10A1B23__INCLUDED_)
