// BongoObject.h: interface for the CBongoObject class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_BONGOOBJECT_H__1F75E7A8_67DA_4D08_9FDB_A1E138755417__INCLUDED_)
#define AFX_BONGOOBJECT_H__1F75E7A8_67DA_4D08_9FDB_A1E138755417__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "BongoKeyframe.h"
#include "BongoConstraintParameterKeyframe.h"

class CBongoObjectKeyframe;
class CBongoKeyframePtr;

class BO_SDK_API_EXP CBongoObject  
{
public:
	CBongoObject();
	CBongoObject(GUID uuidObject);
	CBongoObject(const CRhinoObject*);
	virtual ~CBongoObject();

public:
	bool		IsValid() const;
	bool		IsAnimated() const;
	GUID		Uuid() const;
	void		SetUuid(GUID uuidObject);

public:
	bool		Animate();
	bool		DestroyAllAnimation(bool bApplyChanges = true);

	// Keyframe Iterator
	CBongoKeyframePtr		FirstKeyframe();
	CBongoKeyframePtr		NextKeyframe();

	CBongoKeyframePtr		TweenedValueAtPosition(double dPos, BONGO_KEYFRAME_TYPE type, GUID param) const;

	CBongoKeyframePtr		Keyframe(double dPos, BONGO_KEYFRAME_TYPE type, GUID param) const;
	bool					SetKeyframe(const CBongoKeyframe* pKf, bool bApplyChanges = true);
	bool					DeleteKeyframe(double dPos, GUID type, bool bApplyChanges = true);

	bool		AddConstraint( BONGO_CONSTRAINT_TYPE type, GUID uuidTarget, bool bApplyChanges = true ) ;
	bool		RemoveConstraint( BONGO_CONSTRAINT_TYPE type, GUID uuidTarget, bool bApplyChanges = true );

	bool		Enable(bool bEnable);
	bool		IsEnabled() const;

	bool		ShowPivot() const;
	bool		SetShowPivot( bool flag, bool bRedrawActiveView = true );

	bool		IsPivotLockedToObject() const;
	bool		LockPivotToObject(bool b);

	bool		ShowPivotPath() const;
	bool		SetShowPivotPath(bool flag, bool bRedrawActiveView = true);

	bool		DrawPointOnPivotPath() const;
	bool		SetDrawPointOnPivotPath(bool flag, bool bRedrawActiveView = true);

	bool		Looping() const;
	bool		SetLooping(bool flag, bool bApplyChanges = true);

	bool		LoopingParameters(double& dStart, double& dStop, DWORD& dwRepeat) const;
	bool		SetLoopingParameters(double dStart, double dStop, DWORD dwRepeat, bool ApplyChanges = true);

	bool		Parent(UUID& uuidParent) const;
	bool		SetParent(const CBongoObject*, bool bRedrawActiveView = true);	// Set NULL to make top-level object

	int			NumberOfConstraints() const;

	bool		ConstraintHeading( ON_3dVector& v ) const;
	bool		SetConstraintHeading( ON_3dVector v, bool bApplyChanges = true );

	bool		Transformation( double dPos, ON_Xform& xform ) const;

	bool		Pivot( ON_Xform& xform ) const;
	bool		SetPivot(const ON_Xform, bool bApplyChanges = true);
	bool		GetPivotAtPosition( double dPos, ON_Xform& xform );

	bool		Animation( ON_Xform& xform ) const;
	bool		SetAnimation(const ON_Xform, bool bApplyChanges = true);

	//	Advanced Hierarchy Settings
	bool		SettingsInheritedFromParent() const;
	bool		SetSettingsInheritedFromParent(bool b, bool bApplyChanges = true);

	bool		ActsOnChildsPivot() const;
	bool		SetActsOnChildsPivot(bool b, bool bApplyChanges = true);

	bool		ControlsMovement() const;
	bool		SetControlsMovement(bool b, bool bApplyChanges = true);

	bool		ControlsRotation() const;
	bool		SetControlsRotation(bool b, bool bApplyChanges = true);

	bool		ControlsScaling() const;
	bool		SetControlsScaling(bool b, bool bApplyChanges = true);

	bool		ControlsLooping() const;
	bool		SetControlsLooping(bool b, bool bApplyChanges = true);

	bool		ControlsGeneralParameters() const;
	bool		SetControlsGeneralParameters(bool b, bool ApplyChanges = true);

protected:
	GUID	m_uuidObject;
	LPVOID	m_pBuffer;
	LPVOID	m_pIterate;
};

#endif // !defined(AFX_BONGOOBJECT_H__1F75E7A8_67DA_4D08_9FDB_A1E138755417__INCLUDED_)
