// BongoView.h: interface for the CBongoView class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_BONGOVIEW_H__17084096_2970_4FCE_83B1_E430EF69E6D9__INCLUDED_)
#define AFX_BONGOVIEW_H__17084096_2970_4FCE_83B1_E430EF69E6D9__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "BongoKeyframe.h"
#include "BongoConstraintParameterKeyframe.h"

class CBongoViewKeyframe;
class CBongoKeyframePtr;

class BO_SDK_API_EXP CBongoView  
{
protected:
	CBongoView();
	CBongoView(CRhinoViewport* pView);
	CBongoView(const wchar_t* wszName);

public:

	//Only one CBongoDoc or CBongoView object can be instantiated at any given
	//time.  The pseudo-constructors below enforce this - they will return false
	//if there is already another object around.  In case you are interested,
	//the reason has to do with ensuring the modifications to the document level
	//animation data are undoable.

	static bool ViewObject(CBongoView*& view);
	static bool ViewObject(CBongoView*& view, CRhinoViewport* pViewport);
	static bool ViewObject(CBongoView*& view, const wchar_t* szViewName);

public:	
	virtual ~CBongoView();



public:
	bool		IsValid() const;	//Does this refer to a valid Rhino viewport
	bool		IsAnimated() const;
	bool		Animate();

	ON_wString	ViewName() const;
	void		SetViewName(const wchar_t* wszName);

public:
	bool	DestroyAllAnimation(bool bApplyChanges = true);

	//Keyframe iterator
	//Note - these functions return Smart Pointers - you don't have to destroy them.
	CBongoKeyframePtr		FirstKeyframe() const;
	CBongoKeyframePtr		NextKeyframe() const;

	//param is either the keyframe type (in this case BONGO_VIEW_KEYFRAME) or
	//the object UUID to which the constraint refers (if you specify view_constraint_keyframe
	//for type
	CBongoKeyframePtr		TweenedValueAtPosition(double dPos, BONGO_KEYFRAME_TYPE type, GUID param) const;

	CBongoKeyframePtr		Keyframe(double dPos, BONGO_KEYFRAME_TYPE type, GUID param) const;
	bool					SetKeyframe(const CBongoKeyframe* pKf, bool bApplyChanges = true );
	bool					DeleteKeyframe(double dPos, bool bApplyChanges = true);

	bool	AddConstraint( BONGO_CONSTRAINT_TYPE type, GUID uuidTarget, double dLength = 0.0, bool bApplyChanges = true);
	bool	RemoveConstraint( BONGO_CONSTRAINT_TYPE type, GUID uuidTarget, bool bApplyChanges = true );

	bool	Enable(bool bEnable,bool bApplyChanges = true);
	bool	IsEnabled() const;

	bool	UseSphericalCoordinates() const;
	bool	SetUseSphericalCoordinates(bool b, bool bApplyChanges = true);

	bool	ShowTargetPath() const;
	bool	SetShowTargetPath(bool b, bool bApplyChanges = true);

	bool	ShowCameraPath() const;
	bool	SetShowCameraPath(bool b, bool bApplyChanges = true);

	bool	DrawPointsOnTargetPath() const;
	bool	SetDrawPointsOnTargetPath(bool b, bool bApplyChanges = true);	

	bool	DrawPointsOnCameraPath() const;
	bool	SetDrawPointsOnCameraPath(bool b,bool bApplyChanges = true);

	int		NumberOfConstraints() const;

protected:
	ON_wString m_szViewName;
	LPVOID	m_pBuffer;
	LPVOID	m_pIterate;
};

#endif // !defined(AFX_BONGOVIEW_H__17084096_2970_4FCE_83B1_E430EF69E6D9__INCLUDED_)
