// BBTEmailNotifierMgr.cpp: implementation of the CBBTEmailNotifierMgr class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "BBTEmailNotifierMgr.h"
#include "BBTApp.h"
#include "BBTEmailNotificationDlg.h"
#include "Smtp.h"
#include "LBPFileMgr.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

UINT SendMailThread(LPVOID pParam)
{
    THREADSTRUCT* ts = (THREADSTRUCT*)pParam;
	if(ts && ts->pEmailNotifierMgr)
	{
		ts->pEmailNotifierMgr->SendMail(ts->sEmailSubject, ts->sEmailBody, ts->sFullPathToAttachment);
	}

	delete ts;
	ts = 0;
	
	return 1;
}


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CBBTEmailNotifierMgr::CBBTEmailNotifierMgr()
{
	m_pEmailNotifierHandler = new CBBTEmailNotificationDlg;

	Reset();
}

CBBTEmailNotifierMgr::~CBBTEmailNotifierMgr()
{
	delete m_pEmailNotifierHandler;
	m_pEmailNotifierHandler = 0;
}

bool CBBTEmailNotifierMgr::Initialize(bool bSilent)
{
	Reset();

	if(bSilent || RhinoApp().RunScript(L"_BongoBonusEmailNotification",0))
	{
		if(m_pEmailNotifierHandler->ReadFromRegistry())
		{
			if(m_pEmailNotifierHandler->IsEmailNotificationEnabled())
			{
				if(m_pEmailNotifierHandler->ValidateInput(true))
				{
					m_bEnabled = true;
				}
			}
		}	
	}

	return true;
}

bool CBBTEmailNotifierMgr::Reset()
{
	m_bEnabled = false;
	m_bFirstFrameRendered = true;
	m_iFramesRendered = 0;
	m_iNextFrameToRender = 0;

	return true;
}

bool CBBTEmailNotifierMgr::FrameRendered(const char* szFileName, int iFrameNumber)
{
	m_iFramesRendered++;

	// check if notifications for rendered frames should go out
	if(!m_bEnabled || !(BOOL)m_pEmailNotifierHandler->SendFrameNotification())
	{
		return false;
	}
	
	// store first rendered frame for working out the sequence when to send
	// the next
	if(m_bFirstFrameRendered)
	{
		m_bFirstFrameRendered = false;
		m_iNextFrameToRender = iFrameNumber;
	}

	if(ShouldWeSendNotificationForThisFrame(iFrameNumber))
	{
		CString sText;

		// Okay send info of file
		THREADSTRUCT* ts = new THREADSTRUCT;
		ts->pEmailNotifierMgr = this;
		
		CString sBody = ThePlugIn().LocalizedString(4003) + "\n";
		sText.Format(ThePlugIn().LocalizedString(4004), iFrameNumber);
		sBody += sText + "\n";
		sText.Format(ThePlugIn().LocalizedString(4005),szFileName);
		sBody += sText;

		ts->sEmailBody = sBody;
			
		ts->sEmailSubject = m_pEmailNotifierHandler->Subject();
		
		if((BOOL)m_pEmailNotifierHandler->AttachSingleFrame())
		{
			ts->sFullPathToAttachment = szFileName;
		}

		AfxBeginThread (SendMailThread, ts);
	}


	return true;
}

bool CBBTEmailNotifierMgr::AnimationFinished(const char* szFileName)
{	
	if(!m_bEnabled)
	{
		return false;
	}

	// Send it
	CString sText;

	// Okay send info of file
	THREADSTRUCT* ts = new THREADSTRUCT;
	ts->pEmailNotifierMgr = this;
	
	CString sFileName;
	sFileName = CLBPFileMgr::GetFilenameOnly(szFileName);

	CString sBody = ThePlugIn().LocalizedString(4003);
	sText.Format(ThePlugIn().LocalizedString(4007),sFileName);
	sBody += "\n" + sText;

	ts->sEmailBody = sBody;
		
	ts->sEmailSubject = m_pEmailNotifierHandler->Subject();
	
	AfxBeginThread (SendMailThread, ts);

	return true;
}

bool CBBTEmailNotifierMgr::AnimationCanceled(const char* szFileName)
{
	if(!m_bEnabled)
	{
		return false;
	}

	// Send it
	CString sText;

	// Okay send info of file
	THREADSTRUCT* ts = new THREADSTRUCT;
	ts->pEmailNotifierMgr = this;

	CString sFileName;
	sFileName = CLBPFileMgr::GetFilenameOnly(szFileName);
	
	CString sBody = ThePlugIn().LocalizedString(4003);
	sText.Format(ThePlugIn().LocalizedString(4006),sFileName);
	sBody +=  "\n" + sText;

	ts->sEmailBody = sBody;
		
	ts->sEmailSubject = m_pEmailNotifierHandler->Subject();
	
	AfxBeginThread (SendMailThread, ts);

	return true;
}

bool CBBTEmailNotifierMgr::ShouldWeSendNotificationForThisFrame(int iFrame)
{
	int iFrameSequence = m_pEmailNotifierHandler->NotifyEachXFrame();
	if(iFrameSequence == 0)
	{
		return false;
	}

	if(iFrame == m_iNextFrameToRender)
	{
		m_iNextFrameToRender += iFrameSequence;
		return true;
	}

	return false;
}	

bool CBBTEmailNotifierMgr::SendMail(const char* szSubject, const char* szBody, const char* szAttachment)
{
	if(!m_bEnabled)
	{
		return false;
	}

	// check if smtp address is valid
	CSMTPConnection smtp;
	if(!smtp.Connect(m_pEmailNotifierHandler->SmtpGateway(), m_pEmailNotifierHandler->SmtpPortNumber()))
	{
		RhinoApp().Print(ThePlugIn().LocalizedString(4000).AsMBCSString());
		return false;
	}

	CSMTPMessage msg;
	msg.AddRecipient(CSMTPAddress(m_pEmailNotifierHandler->RecipientAddress()));
	msg.m_From = CSMTPAddress(m_pEmailNotifierHandler->SenderAddress());
	msg.m_sSubject = szSubject;
	msg.AddBody(szBody);

	CSMTPAttachment attachment;

	CString sAttachment = szAttachment;
	if(!sAttachment.IsEmpty())
	{
		attachment.Attach(szAttachment);
		msg.AddAttachment(&attachment);
	}

	if(!smtp.SendMessage(msg))
	{
		DWORD dwError = ::GetLastError();

		CString sError;
		sError.Format(ThePlugIn().LocalizedString(4001),dwError); 
		
		RhinoApp().Print(sError);
		RhinoApp().Print("\n");
		
		return false;
	}

	CString sMsg;
	sMsg.Format(ThePlugIn().LocalizedString(4002), szSubject);
	RhinoApp().Print(sMsg);
	RhinoApp().Print("\n");

	return true;
}

