// BBTGraphicDisplayControlBar.cpp : implementation file
//

#include "stdafx.h"
#include "BBTApp.h"
#include "BBTGraphicDisplayControlBar.h"
#include "LBPRegistryWrapper.h"
#include "BBTGRaphicDisplayDlg.h"


CBBTGraphicDisplayControlBar::CBBTGraphicDisplayControlBar()
{
	SetDrawVerticlyDockedWindowCaptions( false );
	SetDrawVerticlyDockedWindowCaptionText( false );
	SetDrawHorizontallyDockedWindowCaptions( false );
	SetDrawHorizontallyDockedWindowCaptionText( false );
	SetAddHelpBoxToCaption(TRUE);
	SetAddMinimizeBoxToCaption( false );

	m_iTop = 0;
	m_iLeft = 0;
	m_iBottom = 78;
	m_iRight = 500;
	m_dwDockState = AFX_IDW_DOCKBAR_BOTTOM;
	m_dwFloating = 0;
	m_bIsWindowDocked = true;
	m_bVisible = FALSE;

	wcsncpy(m_wszLocalDialogName, ThePlugIn().LocalizedString(6000), BBT_STRING_BUFFER);
}

CBBTGraphicDisplayControlBar::~CBBTGraphicDisplayControlBar()
{
	delete m_pGraphicDisplayDlg;
	m_pGraphicDisplayDlg = 0;

	SaveDialogSettings();
}

BEGIN_MESSAGE_MAP(CBBTGraphicDisplayControlBar, CRmaUiControlBar)
//{{AFX_MSG_MAP(CBBTGraphicDisplayControlBar)
	ON_WM_WINDOWPOSCHANGED()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BOOL CBBTGraphicDisplayControlBar::CreateControlBarX( CFrameWnd *pParentWnd )
{
	USES_CONVERSION;

	DWORD dwStyle = CBRS_TOOLTIPS|CBRS_SIZE_DYNAMIC|CBRS_GRIPPER|CBRS_FLYBY|CBRS_RIGHT;
	UINT nID = ID_DISPLAY_GRAPHIC_CONTROL_BAR;
	
	if( !CreateControlBar(pParentWnd,W2A(m_wszLocalDialogName), nID, dwStyle) )
		return FALSE;

	LoadDialogSettings();

	m_bVisible = FALSE;
	
	// Allow control bar to dock
	EnableDocking(CBRS_ALIGN_BOTTOM | CBRS_ALIGN_TOP);
	
	// Float the control bar so the upper left corner is located in the
	// center of the main Rhino window
	CRect rcWnd;
	pParentWnd->GetWindowRect( rcWnd );

	CPoint pt(m_iLeft,m_iTop);

	if((BOOL)m_dwFloating)	
	{
		pParentWnd->FloatControlBar(this,pt);
	}	
	else	
	{
		pParentWnd->DockControlBar(this, m_dwDockState);
	}
	
	// Display the control bar
	pParentWnd->ShowControlBar( this, m_bVisible, FALSE );
	
	return TRUE;
}

BOOL CBBTGraphicDisplayControlBar::CreateControls()
{
	BOOL bCreated = FALSE;
	
	{
		// Make sure this module state is active so that the dialog
		// will be created using the correct resources

		AFX_MANAGE_STATE( AfxGetStaticModuleState() );

		m_pGraphicDisplayDlg = new CBBTGraphicDisplayDlg(this);
		if(!m_pGraphicDisplayDlg)
			return FALSE;
		
		LPCTSTR lpszTemplateName = m_pGraphicDisplayDlg->DialogStringID();
		
		// Create the dialog
		if(m_pGraphicDisplayDlg->DialogID() > 0)
		{
			bCreated = m_pGraphicDisplayDlg->Create(m_pGraphicDisplayDlg->DialogID(), this);
		}
		else if(lpszTemplateName && lpszTemplateName[0])
		{
			bCreated = m_pGraphicDisplayDlg->Create(lpszTemplateName, this);
		}
	}
	
	if( !bCreated )
	{
		BBTMsgBox(ThePlugIn().LocalizedString(1015));
		return FALSE;
	}

	m_pGraphicDisplayDlg->ShowWindow( SW_SHOW );
	m_pGraphicDisplayDlg->GetClientRect( m_rcDialog );
	m_rcDialog.SetRect(m_iLeft,m_iTop,m_iRight-m_iLeft,m_iBottom-m_iTop);
	
	return TRUE;
}

CRect CBBTGraphicDisplayControlBar::InitialSize()
{
	if(m_pGraphicDisplayDlg)
	{
		return m_rcDialog;
	}

	return CRect( 0, 0, 50, 20 );
}

BOOL  CBBTGraphicDisplayControlBar::ResizeControls( LPRECT lpRect )
{
	// Make the dialog fit the client area of this control bar
	if( m_pGraphicDisplayDlg && ::IsWindow(m_pGraphicDisplayDlg->m_hWnd) )
	{
		CRect rc;
		GetClientRect( rc );
		m_pGraphicDisplayDlg->MoveWindow(rc);
	}
	return TRUE;
}

void CBBTGraphicDisplayControlBar::OnWindowPosChanged(WINDOWPOS FAR* lpwndpos) 
{
	CRmaUiControlBar::OnWindowPosChanged(lpwndpos);
	
	// TODO: Add your message handler code here
	UINT flag = lpwndpos->flags;

	StoreTimelinePos();
	
	if(flag & SWP_HIDEWINDOW)
	{	
		m_pGraphicDisplayDlg->OnWindowClosed();
		m_bVisible = FALSE;
	}
	else if(flag & SWP_SHOWWINDOW)
	{
		m_bVisible = TRUE;
	}
}

void CBBTGraphicDisplayControlBar::StoreTimelinePos()
{
	CFrameWnd* pFrame = GetDockingFrame();
	if(!pFrame)	return;

	CWnd* pMain = CWnd::FromHandle(RhinoApp().MainWnd());
	if(!pMain)	return;

	CRect rcMain, rcFrame;

	m_pGraphicDisplayDlg->GetClientRect(rcFrame);

	m_iBottom = rcFrame.bottom;
	m_iRight = rcFrame.right;

	ClientToScreen(rcFrame);

	WINDOWPLACEMENT Pos;
	pFrame->GetWindowPlacement(&Pos);

	m_iTop = Pos.rcNormalPosition.top;
	m_iLeft = Pos.rcNormalPosition.left;

	pMain->GetClientRect(rcMain);

	if(IsFloating())	
	{
		m_dwFloating = 1;
	}
	else				
	{
		m_dwFloating = 0;

		if(rcFrame.top <= (double)(rcMain.bottom - rcMain.top)/2.0)	
		{
			m_dwDockState = AFX_IDW_DOCKBAR_TOP;
		}
		else
		{
			m_dwDockState = AFX_IDW_DOCKBAR_BOTTOM;
		}
	}

	CPoint ptMid = rcMain.CenterPoint();


	//LynxPrint("Top: %d	Left: %d  Bottom: %d	Right: %d\n", m_iTop, m_iLeft, m_iBottom, m_iRight);

}


BOOL CBBTGraphicDisplayControlBar::LoadDialogSettings()
{
	BOOL rc = TRUE;
	DWORD dwVisible = 0;

	CString sPath = BBT_REGPATH;
	sPath +="Dialog Settings\\Graphical Display\\";

	CLBPRegistryWrapper key;
	DWORD dwTop, dwBottom, dwLeft, dwRight;

	if(rc)	{rc = key.GetDWORDValue(HKEY_CURRENT_USER,sPath,"Floating", m_dwFloating);}
	if(rc)	{rc = key.GetDWORDValue(HKEY_CURRENT_USER,sPath,"Docking State", m_dwDockState);}
	if(rc)	{rc = key.GetDWORDValue(HKEY_CURRENT_USER,sPath,"Top", dwTop);}
	if(rc)	{rc = key.GetDWORDValue(HKEY_CURRENT_USER,sPath,"Left", dwLeft);}
	if(rc)	{rc = key.GetDWORDValue(HKEY_CURRENT_USER,sPath,"Bottom", dwBottom);}
	if(rc)	{rc = key.GetDWORDValue(HKEY_CURRENT_USER,sPath,"Right", dwRight);}
	if(rc)	{rc = key.GetDWORDValue(HKEY_CURRENT_USER,sPath,"Visibility", dwVisible);}

	if(rc)
	{
		m_bVisible = (BOOL)dwVisible;
		m_iTop = (int)dwTop;
		m_iLeft = (int)dwLeft;
		m_iBottom = (int)dwBottom;
		m_iRight = (int)dwRight;
	}

	return rc;
}

BOOL CBBTGraphicDisplayControlBar::SaveDialogSettings()
{
	AdjustWindowSize(m_iLeft, m_iTop, m_iRight, m_iBottom);

	BOOL rc = TRUE;

	CString sPath = BBT_REGPATH;
	sPath +="Dialog Settings\\Graphical Display\\";

	CLBPRegistryWrapper key;

	if(rc)	{rc = key.SetDWORDValue(HKEY_CURRENT_USER,sPath,"Floating", m_dwFloating);}
	if(rc)	{rc = key.SetDWORDValue(HKEY_CURRENT_USER,sPath,"Docking State", m_dwDockState);}
	if(rc)	{rc = key.SetDWORDValue(HKEY_CURRENT_USER,sPath,"Top", m_iTop);}
	if(rc)	{rc = key.SetDWORDValue(HKEY_CURRENT_USER,sPath,"Left", m_iLeft);}
	if(rc)	{rc = key.SetDWORDValue(HKEY_CURRENT_USER,sPath,"Bottom", m_iBottom);}
	if(rc)	{rc = key.SetDWORDValue(HKEY_CURRENT_USER,sPath,"Right", m_iRight);}
	if(rc)	{rc = key.SetDWORDValue(HKEY_CURRENT_USER,sPath,"Visibility", m_bVisible);}

	return rc;
}

void CBBTGraphicDisplayControlBar::RefreshDisplay()
{
	if(m_pGraphicDisplayDlg)
	{
		m_pGraphicDisplayDlg->SetDirtyFlag(true);
	}
}

void CBBTGraphicDisplayControlBar::SetSkipNextSelectionChangedNotification(bool bFlag)
{
	if(m_pGraphicDisplayDlg)
	{
		m_pGraphicDisplayDlg->SetSkipNextSelectionChangedNotification(bFlag);
	}
}

void CBBTGraphicDisplayControlBar::UpdateSliderPos(double dPos)
{
	if(m_pGraphicDisplayDlg)
	{
		m_pGraphicDisplayDlg->UpdateSliderPos(dPos);
	}
}