// BBTGraphicDisplayDlg.cpp : implementation file
//

#include "stdafx.h"
#include "bbtapp.h"
#include "BBTGraphicDisplayDlg.h"
#include "LBPGraphDisplay.h"

#define BBT_UPDATE_DISPLAY_TIMER 001

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CBBTGraphicDisplayDlg dialog


CBBTGraphicDisplayDlg::CBBTGraphicDisplayDlg(CWnd* pParent /*=NULL*/)
	: CRhinoDialog(CBBTGraphicDisplayDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CBBTGraphicDisplayDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT

	m_bDirtyFlag = false;
	m_bSkipNextSelectionChangeNotification = false;
	m_pGraphicDisplay = 0;
	m_modeTransformation = motion_transformation;
}

CBBTGraphicDisplayDlg::~CBBTGraphicDisplayDlg()
{
	delete m_pGraphicDisplay;
	m_pGraphicDisplay = 0;

	KillTimer(BBT_UPDATE_DISPLAY_TIMER);
}

void CBBTGraphicDisplayDlg::DoDataExchange(CDataExchange* pDX)
{
	CRhinoDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CBBTGraphicDisplayDlg)
	DDX_Control(pDX, IDC_COMBO_TRANSFORMATION_MODE, m_comboTransformationMode);
	DDX_Control(pDX, IDC_STATIC_GRAPHIC_PLACE_HOLDER, m_ctrlGraphicPlaceHolder);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CBBTGraphicDisplayDlg, CRhinoDialog)
	//{{AFX_MSG_MAP(CBBTGraphicDisplayDlg)
	ON_WM_TIMER()
	ON_WM_SIZE()
	ON_CBN_SELCHANGE(IDC_COMBO_TRANSFORMATION_MODE, OnSelchangeComboTransformationMode)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CBBTGraphicDisplayDlg message handlers

void CBBTGraphicDisplayDlg::PreSubclassWindow() 
{
	// Remove unwanted styles and add required ones.
	ModifyStyle( DS_MODALFRAME|WS_POPUP|WS_OVERLAPPED|WS_BORDER
		|WS_CAPTION|WS_SYSMENU|WS_MINIMIZEBOX|WS_MAXIMIZEBOX|WS_DLGFRAME,
		WS_CHILD);
	
	ModifyStyleEx( WS_EX_DLGMODALFRAME, 0 );
	
	CRhinoDialog::PreSubclassWindow();
}

void CBBTGraphicDisplayDlg::OnWindowClosed()
{

}

BOOL CBBTGraphicDisplayDlg::OnInitDialog() 
{
	CRhinoDialog::OnInitDialog();
	
	PlaceGraphicDisplayOnDialog();

	InitializeTransformationModeComboBox();

	SetTimer(BBT_UPDATE_DISPLAY_TIMER, 1000, 0);
	

	Register();
	Enable(true);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

bool CBBTGraphicDisplayDlg::PlaceGraphicDisplayOnDialog()
{	
	CRect rcGraphicPlaceHolder;
	m_ctrlGraphicPlaceHolder.GetWindowRect(rcGraphicPlaceHolder);

	ScreenToClient(&rcGraphicPlaceHolder);

	m_pGraphicDisplay = new CLBPGraphDisplay;
	if(!m_pGraphicDisplay->Create(rcGraphicPlaceHolder, this, ID_GRAPHIC_DISPLAY_WINDOW))
	{
		return false;
	}

	return true;
}

bool CBBTGraphicDisplayDlg::InitializeTransformationModeComboBox()
{
	for(int i=6001; i<6004; i++)
	{
		m_comboTransformationMode.AddString(ThePlugIn().LocalizedString(i));
	}

	m_comboTransformationMode.SetCurSel(0);

	return true;	
}

void CBBTGraphicDisplayDlg::OnSize(UINT nType, int cx, int cy) 
{	
	CRhinoDialog::OnSize(nType, cx, cy);

	CRect rc, rcGraphic, rcTransformationMode;
	GetClientRect(rc);

	int iComboWidth = 60;
	int iComboHeight = 15;


	m_comboTransformationMode.SetWindowPos(NULL, rc.right-iComboWidth, rc.top+5, iComboWidth,
		iComboHeight,SWP_NOZORDER);

	m_ctrlGraphicPlaceHolder.GetClientRect(rcGraphic);
	m_comboTransformationMode.GetClientRect(rcTransformationMode);


	// controls overlap
	if(rc.Width() < 200)
	{
		m_comboTransformationMode.ShowWindow(SW_HIDE);

		m_ctrlGraphicPlaceHolder.SetWindowPos(NULL, rc.left+5, rc.top+5, rc.Width()-5, rc.Height()-5,
			SWP_NOZORDER);
	}
	else
	{
		m_comboTransformationMode.ShowWindow(SW_SHOW);

		m_ctrlGraphicPlaceHolder.SetWindowPos(NULL, rc.left+5, rc.top+5, rc.Width()-iComboWidth-10,
			rc.Height()-10, SWP_NOZORDER);
	}

	if(m_pGraphicDisplay && m_pGraphicDisplay->m_hWnd)
	{                                                         // otherwise not created yet
		CRect rc;
		m_ctrlGraphicPlaceHolder.GetWindowRect(rc);
		ScreenToClient(&rc);

		m_pGraphicDisplay->SetWindowPos(NULL, rc.left, rc.top, rc.Width(), rc.Height(), SWP_NOZORDER );
	}
}


void CBBTGraphicDisplayDlg::OnTimer(UINT nIDEvent) 
{
	if(BBT_UPDATE_DISPLAY_TIMER == nIDEvent)
	{
		if(m_bDirtyFlag)
		{
			m_bDirtyFlag = false;
			RefreshDisplay();
		}
	}
	
	CRhinoDialog::OnTimer(nIDEvent);
}

bool CBBTGraphicDisplayDlg::RefreshDisplay()
{
	if(!m_pGraphicDisplay || !IsWindowVisible())
	{
		return false;
	}

	m_pGraphicDisplay->RemoveAllDataSets();

	int iMax = 100;
	CBongoDoc* pBongoDoc = 0;
	if(CBongoDoc::DocumentObject(pBongoDoc) && pBongoDoc)
	{
		iMax = pBongoDoc->TimelineTicks();
	}

	delete pBongoDoc;
	pBongoDoc = 0;


	m_pGraphicDisplay->SetMaxX(iMax);

	bool bIsRotationTransformation = false;

	UUID uuidKeyframeTypeToDisplay(ON_nil_uuid);
	if(m_modeTransformation == rotation_transformation)
	{
		uuidKeyframeTypeToDisplay = BONGO_ROTATION_KEYFRAME;
		bIsRotationTransformation = true;
	}
	else if(m_modeTransformation == scaling_transformation)
	{
		uuidKeyframeTypeToDisplay = BONGO_SCALING_KEYFRAME;
	}
	else if(m_modeTransformation == motion_transformation)
	{
		uuidKeyframeTypeToDisplay = BONGO_MOTION_KEYFRAME;
	}

	m_pGraphicDisplay->SetAsRotation(bIsRotationTransformation);


	CArray<CRhinoObject*, CRhinoObject*> aSelectedObjects;
	if(GetSelectedObjects(aSelectedObjects) > 0)
	{
		CBongoKeyframePtr BongoPtrKf;
		CBongoObjectKeyframe* pBongoKf;

		for(int i=0; i<aSelectedObjects.GetSize(); i++)
		{
			m_pGraphicDisplay->AddDataSet(COLORREF(RGB(255,0,0)));
			m_pGraphicDisplay->AddDataSet(COLORREF(RGB(0,255,0)));
			m_pGraphicDisplay->AddDataSet(COLORREF(RGB(0,0,255)));

			CBongoObject bongo_object(aSelectedObjects[i]->Attributes().m_uuid);
			if(bongo_object.IsValid() && bongo_object.IsAnimated())
			{
				for(BongoPtrKf = bongo_object.FirstKeyframe(); BongoPtrKf;
					BongoPtrKf = bongo_object.NextKeyframe())
				{
					pBongoKf = (CBongoObjectKeyframe*)BongoPtrKf.Pointee();
					if(pBongoKf && pBongoKf->Type() == object_keyframe &&
						pBongoKf->KeyframeType() == uuidKeyframeTypeToDisplay)
					{
						

						GRAPH_POINT pt	(	pBongoKf->Position(),
											pBongoKf->Value().GetPoint().x,
											pBongoKf->Smoothness() / 100.0,
											pBongoKf->Continuity() / 100.0,
											pBongoKf->Bias() / 100.0,
											pBongoKf->EaseIn(),
											pBongoKf->EaseOut()
											);

						m_pGraphicDisplay->AddDataPoint(i+0, pt);

						pt.y = pBongoKf->Value().GetPoint().y;
						m_pGraphicDisplay->AddDataPoint(i+1, pt);


						pt.y = pBongoKf->Value().GetPoint().z;
						m_pGraphicDisplay->AddDataPoint(i+2, pt);

					}
				}
			}
		}
	}

	// now clean up and delete all empty data sets
	for(int i=0; i<m_pGraphicDisplay->DataSetCount(); i++)
	{
		if(m_pGraphicDisplay->GetPointCount(i) <= 0)
		{
			m_pGraphicDisplay->RemoveDataSet(i);
		}
	}

	m_pGraphicDisplay->RedrawWindow();

	return true;
}

void CBBTGraphicDisplayDlg::OnDeselectAllObjects(CRhinoDoc& doc,int count)
{
	OnSelectionChanged();
}

void CBBTGraphicDisplayDlg::OnDeselectObject(CRhinoDoc& doc, const CRhinoObject& object)
{
	OnSelectionChanged();
}

void CBBTGraphicDisplayDlg::OnDeselectObjects(CRhinoDoc& doc,
											  const ON_SimpleArray<const CRhinoObject*>& objects)
{
	OnSelectionChanged();
}

void CBBTGraphicDisplayDlg::OnSelectObject(CRhinoDoc& doc,const CRhinoObject& object)
{
	OnSelectionChanged();
}

void CBBTGraphicDisplayDlg::OnSelectObjects(CRhinoDoc& doc,
											const ON_SimpleArray<const CRhinoObject*>& objects)
{
	OnSelectionChanged();
}

void CBBTGraphicDisplayDlg::OnReplaceObject(CRhinoDoc& doc, CRhinoObject& old_object,
											CRhinoObject& new_object)
{

}

void CBBTGraphicDisplayDlg::OnSelectionChanged()
{
	if(m_bSkipNextSelectionChangeNotification)
	{
		return;
	}

	SetDirtyFlag(true);
}	

bool CBBTGraphicDisplayDlg::GetDirtyFlag()
{
	return m_bDirtyFlag;
}

void CBBTGraphicDisplayDlg::SetDirtyFlag(bool b)
{
	m_bDirtyFlag = b;
}

void CBBTGraphicDisplayDlg::SetSkipNextSelectionChangedNotification(bool bFlag)
{
	m_bSkipNextSelectionChangeNotification = bFlag;
}

void CBBTGraphicDisplayDlg::OnSelchangeComboTransformationMode() 
{
	int iCurSel = m_comboTransformationMode.GetCurSel();
	switch(iCurSel)
	{
	case 0:
		m_modeTransformation = rotation_transformation;
		break;
	case 1:
		m_modeTransformation = scaling_transformation;
	case 2:
		m_modeTransformation = motion_transformation;
	
	}

	SetDirtyFlag(true);
}

void CBBTGraphicDisplayDlg::UpdateSliderPos(double dPos)
{
	if(m_pGraphicDisplay)
	{
		m_pGraphicDisplay->SetSliderPosition(dPos);
	}
}