// BBTMain.cpp: implementation of the CBBTMain class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "BBTApp.h"
#include "BBTMain.h"
#include "BBTEventWatcher.h"
#include "BBTTimelineExtension.h"
#include "BBTGraphicDisplayControlBar.h"

extern CBBTApp theApp;

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif



CBBTMain bbt;


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CBBTMain::CBBTMain()
{
	m_iconBongo = 0;
	m_pBongoEventWatcher = 0;
	m_pTimelineExtension = new CBBTTimelineExtension;
	m_pGraphicDisplayControlBar = 0;
}

CBBTMain::~CBBTMain()
{
	DestroyIcon(m_iconBongo);

	delete m_pBongoEventWatcher;
	m_pBongoEventWatcher = 0;

	delete m_pTimelineExtension;
	m_pTimelineExtension = 0;

	delete m_pGraphicDisplayControlBar;
	m_pGraphicDisplayControlBar = 0;
}

bool CBBTMain::Initialize()
{
	m_iconBongo = ::LoadIcon(theApp.m_hInstance,MAKEINTRESOURCE(IDI_BONGO));
	
	if(!CreateGraphicDisplayControlBar())
	{
		return false;
	}

	return true;
}

bool CBBTMain::WakeUp()
{
	if(!CreateBongoEventWatcher())
	{
		BBTMsgBox(ThePlugIn().LocalizedString(1007), MB_OK|MB_ICONERROR);
		return false;
	}

	return true;
}

HICON CBBTMain::BongoIcon()
{
	return m_iconBongo;
}

CBBTEventWatcher& CBBTMain::BongoEventWatcher()
{
	if(!m_pBongoEventWatcher)
	{
		CreateBongoEventWatcher();
	}

	return *m_pBongoEventWatcher;
}

bool CBBTMain::CreateBongoEventWatcher()
{
	if(!m_pBongoEventWatcher)
	{
		m_pBongoEventWatcher = new CBBTEventWatcher;
		CBongo bongo;
		if(!bongo.RegisterBongoEventWatcher(m_pBongoEventWatcher))
			return false;
	}

	return true;
}

CBBTTimelineExtension& CBBTMain::TimelineExtension()
{
	return *m_pTimelineExtension;
}

CBBTGraphicDisplayControlBar& CBBTMain::GraphicDisplayControlBar()
{
	return *m_pGraphicDisplayControlBar;
}

bool CBBTMain::CreateGraphicDisplayControlBar()
{
	CWnd* pMainWnd = RhinoApp().m_pMainWnd;
	if(!pMainWnd)	return false;

	CFrameWnd* pFrameWnd = (CFrameWnd*)pMainWnd;
	if(!pFrameWnd)	return false;
	
	AFX_MANAGE_STATE(::RhinoApp().RhinoModuleState());

	m_pGraphicDisplayControlBar = new CBBTGraphicDisplayControlBar;
	if(!m_pGraphicDisplayControlBar)	return false;
	
	if(!m_pGraphicDisplayControlBar->CreateControlBarX(pFrameWnd))
	{
		delete m_pGraphicDisplayControlBar;
		m_pGraphicDisplayControlBar = 0;
		return false;
	}

	return true;
}

bool CBBTMain::CloseGraphicDisplayControlBar()
{
	if(m_pGraphicDisplayControlBar)
	{
		CFrameWnd* pFrame = (CFrameWnd*)(m_pGraphicDisplayControlBar->GetTopLevelParent());
		if(pFrame)
		{
			pFrame->ShowControlBar(m_pGraphicDisplayControlBar, false, false);
		}
		
		if(m_pGraphicDisplayControlBar->GetDockingFrame())
		{
			m_pGraphicDisplayControlBar->GetDockingFrame()->RemoveControlBar(
				m_pGraphicDisplayControlBar);
		}
		
		m_pGraphicDisplayControlBar->DestroyWindow();

		delete m_pGraphicDisplayControlBar;
		m_pGraphicDisplayControlBar = 0;

		return true;
	}

	return false;
}

