// BBTObjectWrapper.cpp: implementation of the CBBTObjectWrapper class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "BBTApp.h"
#include "BBTObjectWrapper.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CBBTObjectWrapper::CBBTObjectWrapper()
{

}

CBBTObjectWrapper::~CBBTObjectWrapper()
{

}

bool CBBTObjectWrapper::IsKeyframeAtPos(CBongoObject* pObject, double dPos)
{
	if(!pObject)
		return false;
	
	if(pObject->Keyframe(dPos, object_keyframe, BONGO_MOTION_KEYFRAME))
	{
		return true;
	}
	else if(pObject->Keyframe(dPos, object_keyframe, BONGO_ROTATION_KEYFRAME))
	{
		return true;
	}
	else if(pObject->Keyframe(dPos, object_keyframe, BONGO_SCALING_KEYFRAME))
	{
		return true;
	}
	else if(pObject->Keyframe(dPos, object_keyframe, BONGO_LIGHT_COLOR))
	{
		return true;
	}
	else if(pObject->Keyframe(dPos, object_keyframe, BONGO_GENERAL_GLOSS_COLOR))
	{
		return true;
	}

	return false;
}

int CBBTObjectWrapper::KeyframeCount(CBongoObject* pObject)
{
	if(!pObject || !pObject->IsValid() || !pObject->IsAnimated())
		return 0;

	CBongoKeyframePtr KfPtr;
	int iCount = 0;
	
	for(KfPtr = pObject->FirstKeyframe(); KfPtr; KfPtr = pObject->NextKeyframe())
	{
		iCount++;
	}

	return iCount;
}

bool CBBTObjectWrapper::ValidateObjects(CRhinoGetObject* pGO, bool bIncludeKeyframesCheck /*= true*/)
{
	if(!pGO|| pGO->ObjectCount() <= 0)
		return false;

	CBongoObject BO;

	for(int i=0; i<pGO->ObjectCount(); i++)
	{
		BO.SetUuid(pGO->Object(i).Object()->Attributes().m_uuid);

		if(!BO.IsValid() || !BO.IsAnimated())
			return false;

		if(bIncludeKeyframesCheck && !BO.FirstKeyframe())
			return false;
	}

	return true;
}

bool CBBTObjectWrapper::GetObjectKeyframes(CBongoObject* pObject,
										   CArray<CBongoObjectKeyframe*, CBongoObjectKeyframe*>& aKFs)
{
	if(!pObject)
		return false;

	CBongoKeyframePtr BongoPtrKf;
	CBongoObjectKeyframe* pBongoKf;
	
	for(BongoPtrKf = pObject->FirstKeyframe(); BongoPtrKf; BongoPtrKf = pObject->NextKeyframe() )
	{
		pBongoKf = (CBongoObjectKeyframe*)BongoPtrKf.Pointee();
		if(pBongoKf)
		{
			aKFs.Add(new CBongoObjectKeyframe(*pBongoKf));
		}
	}	

	if(aKFs.GetSize() <= 0)
		return false;

	return true;
}

bool CBBTObjectWrapper::GetObjectKeyframePositions(CBongoObject* pObject, CArray<double, double>& aKFPos)
{
	if(!pObject)
		return false;

	CBongoKeyframePtr BongoPtrKf;
	double dPos;
	bool bAlreadyInList;
	
	for(BongoPtrKf = pObject->FirstKeyframe(); BongoPtrKf; BongoPtrKf = pObject->NextKeyframe() )
	{
		if(BongoPtrKf.Pointee())
		{
			bAlreadyInList = false;
			dPos = BongoPtrKf.Pointee()->Position();

			for(int j=aKFPos.GetSize()-1; j>=0; j--)
			{
				if(dPos == aKFPos[j])
				{
					bAlreadyInList = true;
					break;
				}
			}
			
			if(!bAlreadyInList)
			{
				aKFPos.Add(dPos);
			}
		}

	}	

	if(aKFPos.GetSize() <= 0)
		return false;

	return true;
}

bool CBBTObjectWrapper::DeleteAllKeyframes(CBongoObject* pObject)
{
	if(!pObject)
		return false;

	CArray<double, double> aKFPos;
	if(!GetObjectKeyframePositions(pObject,aKFPos))
		return false;

	for(int i=aKFPos.GetSize()-1; i>=0; i--)
	{
		pObject->DeleteKeyframe(aKFPos[i],BONGO_WHOLE_KEYFRAME,false);	
	}

	aKFPos.RemoveAll();

	return true;
}

void CBBTObjectWrapper::CollectAnimatedObjects(CArray<const CRhinoObject*, const CRhinoObject*>&
														 aObjects)
{
	CBongoObjectIterator it;
	CBongoObject BongoObject;
	CRhinoObject* pObject;
	bool bRet;

	for(bRet = it.First(BongoObject); bRet; bRet = it.Next(BongoObject))
	{
		pObject = SafeObjectLookup(BongoObject.Uuid());
		if(pObject)
		{
			aObjects.Add(pObject);
		}
	}	
}

int CBBTObjectWrapper::SelectedObjectCount(bool bNeedToBeAnimated, bool bNeedToHaveKeyframes)
{
	CRhinoObjectIterator it( CRhinoObjectIterator::normal_objects,
		CRhinoObjectIterator::active_and_reference_objects);

	it.EnableSelectedFilter();
	it.IncludeLights(TRUE);

	int iCount = 0;
	CRhinoObject* pObject = 0;


	for( pObject = it.First(); pObject; pObject = it.Next() )
	{
		if(!pObject->IsSelected()) continue;
	
		if(bNeedToBeAnimated)
		{
			CBongoObject bongo(pObject);
			if(!bongo.IsValid() || !bongo.IsAnimated())
				continue;
		}

		if(bNeedToHaveKeyframes)
		{
			CBongoObject bongo(pObject);
			if(!bongo.IsValid() || !bongo.IsAnimated() || !bongo.FirstKeyframe())
				continue;			
		}

		iCount++;

	}


	return iCount;
}