// BBTRenderAnimationFileCollectionMgr.cpp: implementation of the CBBTRenderAnimationFileCollectionMgr class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "BBTApp.h"
#include "BBTRenderAnimationFileCollectionMgr.h"
#include "LBPRegistryWrapper.h"
#include "LBPSemicolonDelimitedString.h"
#include "LBPFileMgr.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CBBTRenderAnimationFileCollectionMgr::CBBTRenderAnimationFileCollectionMgr()
{
	Reset();
	m_bEnabled = true;
}

CBBTRenderAnimationFileCollectionMgr::~CBBTRenderAnimationFileCollectionMgr()
{
	Reset();
}

void CBBTRenderAnimationFileCollectionMgr::Reset()
{
	m_aFilesToRender.RemoveAll();
	m_bInitialized = false;
	m_iFileCounter = 0;
}

bool CBBTRenderAnimationFileCollectionMgr::Initialize()
{
	if(!m_bEnabled)
	{
		return false;
	}

	Reset();

	if(!GetFileList() || m_aFilesToRender.GetSize() <= 0)
	{
		return false;
	}

	m_bInitialized = true;

	return true;
}

bool CBBTRenderAnimationFileCollectionMgr::GetFileList()
{
	m_aFilesToRender.RemoveAll();

	CString sFileList;

	CLBPRegistryWrapper wrap;
	if(!wrap.GetStringValue(HKEY_CURRENT_USER, BBT_REGPATH, "RenderAnimationFileCollection", sFileList))
		return false;

	if(!sFileList.IsEmpty())
	{

		CString sFile, sExt;

		CLBPSemicolonDelimitedString StringExtractor(sFileList);
		for(int i= 0; StringExtractor.GetItemByPosition(i, sFile); i++)
		{
			sExt = CLBPFileMgr::GetExtension(sFile);
			if(sExt.CompareNoCase("3dm") == 0 && CLBPFileMgr::IsExistingFile(sFile))
			{
				m_aFilesToRender.Add(sFile);
			}
		}
	}

	return true;
}

bool CBBTRenderAnimationFileCollectionMgr::FirstFile(CString& sFile)
{
	if(!m_bInitialized)
	{
		return false;
	}

	m_iFileCounter = 0;
	
	if(m_aFilesToRender.GetSize() <= 0)
	{
		return false;
	}

	sFile = m_aFilesToRender[0];

	return true;
}

bool CBBTRenderAnimationFileCollectionMgr::GetNextFile(CString& sFile)
{
	if(!m_bInitialized)
	{
		return false;
	}

	m_iFileCounter++;

	if(m_iFileCounter >= m_aFilesToRender.GetSize())
	{
		return false;
	}

	sFile = m_aFilesToRender[m_iFileCounter];

	return true;
}

void CBBTRenderAnimationFileCollectionMgr::Enable(bool bEnable)
{
	m_bEnabled = bEnable;
}

bool CBBTRenderAnimationFileCollectionMgr::IsEnbaled()
{
	return m_bEnabled;
}

bool CBBTRenderAnimationFileCollectionMgr::Render()
{
	if(!m_bEnabled || !m_bInitialized || m_aFilesToRender.GetSize() <= 0 ||
		m_iFileCounter >= m_aFilesToRender.GetSize())
	{
		return false;
	}


	CRhinoDoc* pDoc = RhinoApp().ActiveDoc();
	if(pDoc)
	{
		BOOL bIsModified = pDoc->IsModified();
		if(bIsModified)
		{
			pDoc->SetModifiedFlag(FALSE);
		}
	}


	CString sFile = m_aFilesToRender[m_iFileCounter];
	m_iFileCounter++;

	ON_wString wCommand;
	wCommand.Format("! -_Open \"%s\" ",sFile);

	bool bRet = RhinoApp().RunScript(wCommand,0);
	if(bRet)
	{
		bRet = RhinoApp().RunScript(L"! -_BongoRenderAnimation _Enter",0);
	}
	else
	{
		Render();
	}

	return true;
}