#include "BBTUtilities.h"
#include "StdAfx.h"
#include "ping.h"



void BBTBeep()
{
#ifdef _DEBUG
    Beep(0x300,10);
#endif
}

int BBTMsgBox(const char* szMessage, UINT uType)
{
	HWND hwnd = RhinoApp().MainWnd();
	CString sTitle = "Bongo Bonus Tools";

	return ::MessageBox( hwnd, szMessage, sTitle, uType );	
}

CRhinoObject* SafeObjectLookup(UUID ObjectUuid)
{
	CRhinoObject* pObject = RhinoApp().ActiveDoc()->LookupObject(ObjectUuid);
	if(pObject)	{return pObject;}
	
#ifdef _DEBUG
	
	CRhinoObjectIterator it(CRhinoObjectIterator::undeleted_objects,CRhinoObjectIterator::active_and_reference_objects);
	it.IncludeLights();
	for(pObject = it.First(); pObject; pObject = it.Next())
	{
		if(ON_UuidCompare(ObjectUuid, pObject->Attributes().m_uuid) == 0)
			return pObject;
	}
	
#endif
	
	return NULL;
}

bool GetCommandLineInteger(DWORD dwStringID, int& iNumber, int iMin /*= 0*/, int iMax /*= 1*/,
						   int iDefault /*= 0*/, bool bUseBounds /*= true*/, bool bUseDefault /*= true*/)
{
	WCHAR wsz_Prompt[BBT_STRING_BUFFER];
	wcsncpy(wsz_Prompt, ThePlugIn().LocalizedString(dwStringID),BBT_STRING_BUFFER);

	// Pick origin
	CRhinoGetInteger gi;
	gi.SetCommandPrompt(wsz_Prompt);
	
	if(bUseBounds)
	{
		gi.SetLowerLimit(iMin);
		gi.SetUpperLimit(iMax);
	}

	if(bUseDefault)		
	{
		gi.SetDefaultInteger(iDefault);
		gi.AcceptNothing();
	}
		
	gi.GetInteger();

	switch (gi.Result())
	{
	case CRhinoGet::cancel:
		return false;

	case CRhinoGet::nothing:
		iNumber = iDefault;
		return true;
		
	case CRhinoGet::number:
		iNumber = gi.Number();
		return true;
	}

	return false;
}

bool IsValidEmailAddress(const char* sAddress) 
{ 
	CString sEmailAddy = sAddress;
	if(sEmailAddy.IsEmpty())	
		return false;

	int  iIndex = sEmailAddy.Find('@');
	if(iIndex < 1)
		return false;

	CString sDomain = sEmailAddy.Mid(iIndex+1);

	sEmailAddy = sEmailAddy.Mid(iIndex+1);
	iIndex = sEmailAddy.Find('.');
	if(iIndex < 1)
		return false;

	sEmailAddy = sEmailAddy.Mid(iIndex+1);
	if(sEmailAddy.GetLength() < 1)
		return false;

	return IsValidDomainName(sDomain); 
}

bool IsValidDomainName(const char* sAddress)
{
	CString sSmtp = sAddress;
	if(sSmtp.IsEmpty())	return FALSE;

	CPing p1;
	CPingReply pr1;
	if (p1.Ping2(sAddress, pr1))
	{
		hostent* phostent = gethostbyaddr((char *)&pr1.Address.S_un.S_addr, 4, PF_INET);
		if(!phostent)	{phostent = gethostbyname(sAddress);}
		if(!phostent)
		{
			int iError = WSAGetLastError();	
			return false;
		}

		return true;
	}

	return false;
}

int GetSelectedObjects(CArray<CRhinoObject*, CRhinoObject*>& aObjects)
{
	CRhinoObjectIterator it( CRhinoObjectIterator::normal_objects,
		CRhinoObjectIterator::active_and_reference_objects  );

	it.EnableSelectedFilter();
	it.IncludeLights(TRUE);

	CRhinoObject* pObject = NULL;
	for( pObject = it.First(); pObject; pObject = it.Next() )
	{
		aObjects.Add( pObject );
	}

	return aObjects.GetSize();	
}

bool AdjustWindowSize(int& iLeft, int& iTop, int iWidth, int iHeight)
{
	CRect rcDesktop;

    SystemParametersInfo(SPI_GETWORKAREA,0, rcDesktop, 0);

	if(iLeft + iWidth < rcDesktop.left + 10)	// out of left bound
	{
		iLeft += 10;
	}
	else if(iLeft > rcDesktop.right-20)		// out of right bound
	{
		iLeft -= 20;		
	}

	if(iTop + iHeight < rcDesktop.top + 10)
	{
		iTop += 10;
	}
	else if(iTop > rcDesktop.bottom - 20)
	{
		iTop -= 20;
	}


	return true;
}

bool CompareDouble(double d1, double d2, const int iPrecision)
{
	double d3 = fabs(d1 - d2);
	double dFuzz = pow(10,-iPrecision);
	
	if (d3 < dFuzz) return TRUE;
	return FALSE;
}	

ON_3dVector CrossProduct(ON_3dVector& v1, ON_3dVector& v2)
{
	ON_3dVector vr;
	
	vr.x = (v1.y*v2.z)-(v1.z*v2.y);
	vr.y = (v1.z*v2.x)-(v1.x*v2.z);
	vr.z = (v1.x*v2.y)-(v1.y*v2.x);
	
	return vr;
}

bool vector_equal(const ON_3dVector v1, const ON_3dVector v2, const int iPrecision)
{
	if(!CompareDouble(v1.x,v2.x,iPrecision)) return FALSE;
	if(!CompareDouble(v1.y,v2.y,iPrecision)) return FALSE;
	if(!CompareDouble(v1.z,v2.z,iPrecision)) return FALSE;
	
	return TRUE;	
}

double GetAngleBetweenTwoVectors( ON_3dVector VecOne, ON_3dVector VecTwo, ON_3dVector vNormal )
{
	double Numerator,Denominator,Radians,Result=0;
	
	if(vector_equal(VecOne,VecTwo,5))
	{
		return 0;
	}
	
	Radians = 1;
	
	Numerator = VecOne*VecTwo;
	Denominator = VecOne.Length() * VecTwo.Length();
	
	vNormal.Unitize();
	
	ON_3dVector vCross = CrossProduct(VecOne,VecTwo);
	vCross.Unitize();
	
	if(vector_equal(vCross,ON_3dVector(0,0,0),9))
	{
		if(CompareDouble(Numerator,1.0,6))			return 0;
		else if(CompareDouble(Numerator,-1.0,6))	return 180;
	}
	
	Result = (Numerator / Denominator);

	if(Result > 1.0)		{Result = 1.0;}
	else if(Result < -1.0)	{Result = -1.0;}
	
	if(CompareDouble( Result, -1.0, 6 ))		return 180.0;
	
	Result = acos(Result);
	
	Result = Result * 180 / PI;
	
	// Check if vCross is parallel or anti parallel to normal vector.
	// If anti parallel Angle = 360 - Angle
	double dDot = vCross * vNormal;
	
	if( CompareDouble( dDot, -1.0, 6 ))
		Result = 360.0 - Result;
	
	return Result;
}

double GetAngleBetweenTwoVectors(ON_3dVector VecOne, ON_3dVector VecTwo)
{
	double Numerator,Denominator,Radians,Result=0;
	
	//Radians = PI/180;
	Radians = 1;
	
	Numerator = VecOne*VecTwo;
	Denominator = VecOne.Length() * VecTwo.Length();
	
	ON_3dVector vCross = CrossProduct(VecOne,VecTwo);
	
	// Error checking. ACOS function only accepts values between 1 and -1
	
	vCross.x = fabs(vCross.z);
	vCross.y = fabs(vCross.z);
	vCross.z = fabs(vCross.z);
	
	if(vector_equal(vCross,ON_3dVector(0,0,0),9))
	{
		if(CompareDouble(Numerator,1.0,6))			return 0;
		else if(CompareDouble(Numerator,-1.0,6))	return 180;
	}
	
	//if(Denominator == 0)	{return 0;}
	//else if(Numerator == Denominator)	{return 0;}
	
	Result = (Numerator / Denominator);

	if(Result > 1.0)		{Result = 1.0;}
	else if(Result < -1.0)	{Result = -1.0;}

	if(CompareDouble( Result, -1.0, 6 ))		return 180.0;
	
	Result = acos(Result);
	
	Result = Result * 180 / PI;
	
	return Result;
}

bool GetCameraYawAndPitchAngles(const ON_3dVector vDirection, double& dYaw, double& dPitch)
{
	ON_3dVector vFlatDirection(vDirection);
	vFlatDirection.z = 0.0;

	//ON_3dVector vNormal = CrossProduct(vDirection, vFlatDirection);

	dYaw = GetAngleBetweenTwoVectors(ON_3dVector::UnitVector(1), vFlatDirection, ON_3dVector::UnitVector(2));
	dPitch = GetAngleBetweenTwoVectors(vDirection, vFlatDirection);

	if(vDirection.z < 0)
	{
		dPitch = -dPitch;
	}

	return true;
}